"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rx = require("rx");
const log4js = require("log4js");
const request = require("request");
const logger = log4js.getLogger('spiderjs-sdk-admin');
class Engine {
    constructor(url, accesskey) {
        this.url = url;
        this.accesskey = accesskey;
    }
    connect() {
        return this
            .post('/admin/accesskey', { oid: this.accesskey.oid, key: this.accesskey.key })
            .map((token) => {
            this.token = token;
            return this;
        });
    }
    createApp(app) {
        return this.post('/engine/app', { app });
    }
    deleteApp(oid) {
        return this.delete('/engine/app', { oid });
    }
    getApp(oid) {
        return this.get('/engine/app', { oid });
    }
    deploy(oid, accesskey) {
        return this.post('/engine/executor', { oid, accesskey });
    }
    undeploy(oid) {
        return this.delete('/engine/executor', { oid });
    }
    getExecutor(oid) {
        return this.get('/engine/executor', { oid });
    }
    createTask(oid) {
        return this.post('/engine/task', { oid });
    }
    closeTask(oid) {
        return this.delete('/engine/task', { oid });
    }
    getTask(oid) {
        return this.get('/engine/task', { oid });
    }
    call(call) {
        return this.post('/engine/call', { call });
    }
    post(method, params) {
        const headers = {};
        headers.Authorization = `JWT ${this.token}`;
        return rx.Observable.create((ob) => {
            request.post({
                body: params,
                headers,
                json: true,
                timeout: 6000,
                url: `${this.url}${method}`,
            }, (error, response, resp) => {
                if (error) {
                    ob.onError(error);
                    return;
                }
                if (resp.code !== 'SUCCESS') {
                    ob.onError(resp);
                    return;
                }
                ob.onNext(resp.data);
                ob.onCompleted();
            });
        });
    }
    delete(method, params) {
        const headers = {};
        headers.Authorization = `JWT ${this.token}`;
        return rx.Observable.create((ob) => {
            request.del({
                body: params,
                headers,
                json: true,
                url: `${this.url}${method}`,
            }, (error, response, resp) => {
                if (error) {
                    ob.onError(error);
                    return;
                }
                if (resp.code !== 'SUCCESS') {
                    ob.onError(resp);
                    return;
                }
                ob.onNext(resp.data);
                ob.onCompleted();
            });
        });
    }
    get(method, params) {
        const headers = {};
        headers.Authorization = `JWT ${this.token}`;
        return rx.Observable.create((ob) => {
            request.get({
                headers,
                json: true,
                qs: params,
                url: `${this.url}${method}`,
            }, (error, response, resp) => {
                if (error) {
                    ob.onError(error);
                    return;
                }
                if (resp.code !== 'SUCCESS') {
                    ob.onError(resp);
                    return;
                }
                ob.onNext(resp.data);
                ob.onCompleted();
            });
        });
    }
}
exports.Engine = Engine;
//# sourceMappingURL=engine.js.map