"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const project_1 = require("./project");
const rx = require("rx");
const path = require("path");
const process = require("process");
const log4js = require("log4js");
exports.logger = log4js.getLogger('spiderjs');
const project = new project_1.default(process.cwd());
function dodeploy() {
    return project
        .load()
        .flatMap(() => {
        return project.undeploy();
    })
        .flatMap(() => {
        return project.deploy();
    });
}
function deploy() {
    dodeploy()
        .subscribe(() => {
        exports.logger.info(`deploy spiderjs[${project.spiderjs.executor}] -- success`);
    }, (error) => {
        exports.logger.info(`deploy spiderjs -- failed`, error);
    });
}
exports.deploy = deploy;
function donewtask(name) {
    return project
        .load()
        .flatMap(() => {
        return project.createTask(name);
    });
}
function newtask(name) {
    if (!name) {
        exports.logger.error(`expect task name`);
        return;
    }
    donewtask(name)
        .subscribe(() => {
        exports.logger.info(`create task[${name}] -- success`);
    }, (error) => {
        exports.logger.info(`create task[${name}] -- failed`, error);
    });
}
exports.newtask = newtask;
function dodeltask(name) {
    return project
        .load()
        .flatMap(() => {
        return project.closeTask(name);
    });
}
function deltask(name) {
    if (!name) {
        exports.logger.error(`expect task name`);
        return;
    }
    dodeltask(name)
        .subscribe(() => {
        exports.logger.info(`delete task[${name}] -- success`);
    }, (error) => {
        exports.logger.info(`delete task[${name}] -- failed`, error);
    });
}
exports.deltask = deltask;
function task(name) {
    if (!name) {
        exports.logger.error(`expect task name`);
        return;
    }
    project
        .load()
        .flatMap(() => {
        return rx.Observable
            .timer(0, project.spiderjs.flush * 1000);
    })
        .flatMap(() => {
        exports.logger.info(`flush task[${name}] status ...`);
        return project.getTask(name);
    })
        .map((task) => {
        exports.logger.info(`
                    createTime: ${task.createTime}
                    completeTime: ${task.completeTime}
                    calls: ${task.calls}
                    results: ${task.results}
                    erros: ${task.errors}
                `);
        return {};
    })
        .subscribe(() => {
        exports.logger.info(`flush task[${name}] -- success`);
    }, (error) => {
        exports.logger.info(`flush task[${name}] -- failed`, error);
    });
}
exports.task = task;
function dorun(name) {
    return project
        .load()
        .flatMap(() => {
        if (!project.spiderjs.app.oid) {
            exports.logger.error(`run deploy command first`);
            return rx.Observable.empty();
        }
        if (!project.spiderjs.executor) {
            exports.logger.error(`run deploy command first`);
            return rx.Observable.empty();
        }
        if (!project.spiderjs.task || !project.spiderjs.task[name]) {
            exports.logger.error(`run newtask command first`);
            return rx.Observable.empty();
        }
        const mainfunc = require(path.join(process.cwd(), './src/main')).main;
        return rx.Observable.create((ob) => {
            const context = {
                call: (lambda, params) => {
                    ob.onNext({
                        app: project.spiderjs.app.oid,
                        executor: project.spiderjs.executor,
                        lambda,
                        params: JSON.stringify(params),
                        task: project.spiderjs.task[name],
                    });
                },
                exit: () => {
                    ob.onCompleted();
                },
            };
            mainfunc(context);
        });
    })
        .flatMap((call) => {
        return project.getEngine().flatMap((engine) => {
            return engine.call(call).retry(60);
        });
    });
}
function run(name) {
    if (!name) {
        exports.logger.error(`expect task name`);
        return;
    }
    dorun(name)
        .subscribe(() => {
        exports.logger.info(`send task[${name}] call -- success`);
    }, (error) => {
        exports.logger.info(`send task[${name}] call -- failed`, error);
    });
}
exports.run = run;
function runf(name) {
    if (!name) {
        exports.logger.error(`expect task name`);
        return;
    }
    dodeploy()
        .flatMap(() => {
        if (!project.spiderjs.task) {
            project.spiderjs.task = {};
        }
        if (project.spiderjs.task[name]) {
            return dodeltask(name).flatMap(() => donewtask(name));
        }
        return donewtask(name);
    })
        .flatMap(() => {
        return dorun(name);
    })
        .subscribe(() => {
        exports.logger.info(`send task[${name}] call -- success`);
    }, (error) => {
        exports.logger.info(`send task[${name}] call -- failed`, error);
    }, () => {
        exports.logger.info(`runf -- completed`);
    });
}
exports.runf = runf;
//# sourceMappingURL=cmd.js.map