import {
    IAccessKey, IAccount, IAdmin,
} from './api';

import rx = require('rx');
import log4js = require('log4js');
import request = require('request');
const logger = log4js.getLogger('spiderjs-sdk-admin');

export class Admin implements IAdmin {
    private token: string;
    constructor(private url: string) {
    }

    public login(accout: IAccount): rx.Observable<{}> {
        return this
            .post<string>('/admin/login', accout)
            .map((token) => {
                this.token = token;
                return token;
            });
    }

    public accessKeyOfEngine(): rx.Observable<IAccessKey> {
        return this
            .post<IAccessKey>('/admin/accesskey/engine');
    }

    public accessKeyOfAgent(): rx.Observable<IAccessKey> {
        return this
            .post<IAccessKey>('/admin/accesskey/agent');
    }

    private post<T>(method: string, params?: any): rx.Observable<T> {

        const headers: any = {};

        if (this.token) {

            headers.Authorization = `JWT ${this.token}`;
        }

        return rx.Observable.create<T>((ob) => {
            request.post({
                body: params,
                headers,
                json: true,
                url: `${this.url}${method}`,
            }, (error, response, resp) => {
                if (error) {
                    ob.onError(error);
                    return;
                }

                if (resp.code !== 'SUCCESS') {
                    ob.onError(resp);
                    return;
                }

                ob.onNext(resp.data);

                ob.onCompleted();
            });
        });
    }
}
