import rx = require('rx');

export interface IAccount {
    username: string;
    password: string;
}

export interface ILambda {
    name: string;  // 执行闭包名称
    script: string; // 执行闭包JS代码
}

export interface IApp {
    oid?: string; // APP对象OID
    name: string; // APP名称
    createdby?: string; // 创建者OID
    description: string; // 描述
    lambda: ILambda[];
    updateTime?: string;
}

export interface IExecutor {
    oid: string; // 分布式执行引擎对象ID
    createdby: string; // 创建者OID
    app: IApp; // 执行引擎对应的程序
    agents: string[]; // 执行引擎部署的代理节点
    updateTime: string;
}

export interface IAdmin {
    login(accout: IAccount): rx.Observable<{}>;
}

export interface IEngine {
    connect(): rx.Observable<IEngine>;
    createApp(app: IApp): rx.Observable<string>;
    deleteApp(oid: string): rx.Observable<{}>;
    createTask(oid: string): rx.Observable<string>;
    getTask(oid: string): rx.Observable<ITask>;
    closeTask(oid: string): rx.Observable<{}>;
    getApp(oid: string): rx.Observable<IApp>;
    deploy(oid: string, accesskey: string): rx.Observable<string>;
    undeploy(oid: string): rx.Observable<{}>;
    getExecutor(oid: string): rx.Observable<IExecutor>;
    call(call: ICall): rx.Observable<ITask>;
}

export interface IAccessKey {
    oid: string;
    key: string;
    user: string;
    roles: Role[];
}

export type Role = 'Agent' | 'User' | 'Admin' | 'Engine';

export interface ITask {
    oid: string;
    createdby: string;
    executor: IExecutor;
    createTime: string;
    completeTime: string;
    entry: string;
    params: string;
    status: TaskStatus;
    calls: number;
    results: number;
    errors: number;
}

export type TaskStatus = 'RUNNING' | 'COMPLETED';

export interface ICallStack {
    oid: string; // 所属调用对象
    executor: string; // 所属执行引擎
    agent: string; // 所属代理
    starttime: string; // 开始时间
    endtime: string; // 执行结束时间
}

export interface ICall {
    // oid: string; // Call对象OID
    app: string; // 所属应用对象OID
    executor: string; // 调用目标执行引擎对象OID
    task: string; // 所属任务对象OID
    lambda: string; // lamebda name
    timestamp?: string; // 调用产生的时间点
    params: string; // 字符串化的JSON数据
    trace?: ICallStack[]; // 调用栈跟踪
}
