import Project from './project';

import rx = require('rx');
import fs = require('fs');
import path = require('path');
import process = require('process');
import log4js = require('log4js');
import api = require('./../api');

export const logger = log4js.getLogger('spiderjs');
const project = new Project(process.cwd());

function dodeploy(): rx.Observable<{}> {
    return project
        .load()
        .flatMap(() => {
            return project.undeploy();
        })
        .flatMap(() => {
            return project.deploy();
        });
}

export function deploy() {
    dodeploy()
        .subscribe(() => {
            logger.info(`deploy spiderjs[${project.spiderjs.executor}] -- success`);
        }, (error) => {
            logger.info(`deploy spiderjs -- failed`, error);
        });
}

function donewtask(name: string): rx.Observable<{}> {
    return project
        .load()
        .flatMap(() => {
            return project.createTask(name);
        });
}

export function newtask(name: string) {
    if (!name) {
        logger.error(`expect task name`);
        return;
    }
    donewtask(name)
        .subscribe(() => {
            logger.info(`create task[${name}] -- success`);
        }, (error) => {
            logger.info(`create task[${name}] -- failed`, error);
        });
}

function dodeltask(name: string): rx.Observable<{}> {
    return project
        .load()
        .flatMap(() => {
            return project.closeTask(name);
        });
}

export function deltask(name: string) {
    if (!name) {
        logger.error(`expect task name`);
        return;
    }

    dodeltask(name)
        .subscribe(() => {
            logger.info(`delete task[${name}] -- success`);
        }, (error) => {
            logger.info(`delete task[${name}] -- failed`, error);
        });
}

export function task(name: string) {
    if (!name) {
        logger.error(`expect task name`);
        return;
    }

    project
        .load()
        .flatMap(() => {
            return rx.Observable
                .timer(0, project.spiderjs.flush * 1000);
        })
        .flatMap(() => {
            logger.info(`flush task[${name}] status ...`);
            return project.getTask(name);
        })
        .map((task) => {
            logger.info(`
                    createTime: ${task.createTime}
                    completeTime: ${task.completeTime}
                    calls: ${task.calls}
                    results: ${task.results}
                    erros: ${task.errors}
                `);

            return {};
        })
        .subscribe(() => {
            logger.info(`flush task[${name}] -- success`);
        }, (error) => {
            logger.info(`flush task[${name}] -- failed`, error);
        });
}

function dorun(name: string): rx.Observable<{}> {
    return project
        .load()
        .flatMap(() => {

            if (!project.spiderjs.app.oid) {
                logger.error(`run deploy command first`);
                return rx.Observable.empty<any>();
            }

            if (!project.spiderjs.executor) {
                logger.error(`run deploy command first`);
                return rx.Observable.empty<any>();
            }

            if (!project.spiderjs.task || !project.spiderjs.task[name]) {
                logger.error(`run newtask command first`);
                return rx.Observable.empty<any>();
            }

            const mainfunc = require(path.join(process.cwd(), './src/main')).main;

            return rx.Observable.create<api.ICall>((ob) => {

                const context = {
                    call: (lambda: string, params: any) => {
                        ob.onNext({
                            app: project.spiderjs.app.oid,
                            executor: project.spiderjs.executor,
                            lambda,
                            params: JSON.stringify(params),
                            task: project.spiderjs.task[name],
                        });
                    },
                    exit: () => {
                        ob.onCompleted();
                    },
                };

                mainfunc(context);
            });
        })
        .flatMap((call) => {
            return project.getEngine().flatMap((engine) => {
                return engine.call(call).retry(60);
            });
        });
}

export function run(name: string) {
    if (!name) {
        logger.error(`expect task name`);
        return;
    }

    dorun(name)
        .subscribe(() => {
            logger.info(`send task[${name}] call -- success`);
        }, (error) => {
            logger.info(`send task[${name}] call -- failed`, error);
        });
}

export function runf(name: string) {
    if (!name) {
        logger.error(`expect task name`);
        return;
    }

    dodeploy()
        .flatMap(() => {
            if (!project.spiderjs.task) {
                project.spiderjs.task = {};
            }

            if (project.spiderjs.task[name]) {
                return dodeltask(name).flatMap(() => donewtask(name));
            }

            return donewtask(name);
        })
        .flatMap(() => {
            return dorun(name);
        })
        .subscribe(() => {
            logger.info(`send task[${name}] call -- success`);
        }, (error) => {
            logger.info(`send task[${name}] call -- failed`, error);
        }, () => {
            logger.info(`runf -- completed`);
        });
}
