import {
    only, skip, slow, suite, test, timeout,
} from 'mocha-typescript';

import {
    Admin, Engine, IAccessKey, IApp, IEngine, ILambda,
} from '../src';

import log4js = require('log4js');
import crypto = require('crypto');
import assert = require('assert');

const admin = new Admin('http://localhost:8080');

let engine: IEngine;

const app: IApp = {
    description: '',
    lambda: [] as ILambda[],
    name: '',
};

let executor: string;

let agent: IAccessKey;

@suite('engine test')
class EngineTest {
    @test('login test')
    public connectTest(done: any) {
        admin.login({
            password: 'admin',
            username: 'admin',
        }).subscribe(() => {
            done();
        });
    }

    @test('create access key of engine')
    public accessKeyOfEngine(done: any) {
        admin
            .accessKeyOfEngine()
            .subscribe((key) => {
                engine = new Engine('http://localhost:8080', key);
                done();
            }, (error) => {
                done(error);
            });
    }

    @test('create access key of agent')
    public accessKeyOfAgent(done: any) {
        admin
            .accessKeyOfAgent()
            .subscribe((key) => {
                agent = key;
                done();
            }, (error) => {
                done(error);
            });
    }

    @test('connect engine')
    public connectEngine(done: any) {
        engine
            .connect()
            .subscribe(() => {
                done();
            }, (error) => {
                done(error);
            });
    }

    @test('create app')
    public createApp(done: any) {
        engine
            .createApp(app)
            .subscribe((oid) => {
                app.oid = oid;
                done();
            }, (error) => {
                done(error);
            });
    }

    @test('get app')
    public getApp(done: any) {
        engine
            .getApp(app.oid as string)
            .subscribe(() => {
                done();
            }, (error) => {
                done(error);
            });
    }

    @test('create executor')
    public createExecutor(done: any) {

        return engine
            .deploy(app.oid as string, agent.oid)
            .subscribe((oid) => {
                executor = oid;
                done();
            }, (error) => {
                done(error);
            });

    }

    @test('get executor')
    public getExecutor(done: any) {
        engine
            .getExecutor(executor)
            .subscribe(() => {
                done();
            }, (error) => {
                done(error);
            });
    }

    @test('create task')
    public createTask(done: any) {
        engine
            .createTask(executor)
            .subscribe(() => {
                done();
            }, (error) => {
                done(error);
            });
    }

    @test('delete executor')
    public deleteExecutor(done: any) {
        engine
            .undeploy(executor)
            .subscribe((oid) => {
                done();
            }, (error) => {
                done(error);
            });
    }

    @test('delete app')
    public deleteApp(done: any) {
        engine
            .deleteApp(app.oid as string)
            .subscribe(() => {
                done();
            }, (error) => {
                done(error);
            });
    }

    // @test('create app')
    // public createApp2(done: any) {
    //     engine
    //         .createApp(app)
    //         .subscribe((oid) => {
    //             app.oid = oid;
    //             done();
    //         }, (error) => {
    //             done(error);
    //         });
    // }

    // @test('create executor')
    // public createExecutor2(done: any) {

    //     return engine
    //         .deploy(app.oid as string, agent.oid)
    //         .subscribe((oid) => {
    //             executor = oid;
    //             done();
    //         }, (error) => {
    //             done(error);
    //         });

    // }

}
